import { isMomentObject, objectKeys, isDate, isNil, isMongooseObject, hasJsonMethod, nameOf, classOf, isObject, isClass, isCollection, isString, isObjectID, ancestorsOf, getRandomId } from "@tsed/core";
class JsonMapperCompiler {
  constructor() {
    this.cache = /* @__PURE__ */ new Map(), this.mappers = {}, this.schemes = {}, this.constructors = {}, this.globals = {
      isCollection,
      isClass,
      isObject,
      classOf,
      nameOf,
      hasJsonMethod,
      isMongooseObject,
      isNil,
      isDate,
      objectKeys,
      isMomentObject
    }, this.addGlobal("alterIgnore", this.alterIgnore.bind(this)), this.addGlobal("alterValue", this.alterValue.bind(this)), this.addGlobal("execMapper", this.execMapper.bind(this)), this.addGlobal("compileAndMap", this.map.bind(this));
  }
  addTypeMapper(e, t) {
    const s = nameOf(e);
    return this.cache.set(
      e,
      (/* @__PURE__ */ new Map()).set("typeMapper", {
        id: s,
        fn: t
      })
    ), this.mappers[s] = t, this;
  }
  removeTypeMapper(e) {
    const t = this.cache.get(e);
    if (t) {
      const { id: s } = t.get("typeMapper");
      delete this.mappers[s], this.cache.delete(e);
    }
  }
  addGlobal(e, t) {
    return this.globals[e] = t, this;
  }
  eval(mapper, { id, groupsId, model }) {
    this.addGlobal("cache", this.cache);
    const { globals, schemes } = this, injectGlobals = Object.keys(globals).map((e) => `const ${e} = globals.${e};`).join(`
`);
    eval(`${injectGlobals};

    cache.get(model).set(groupsId, { id: '${id}', fn: ${mapper} })`);
    const store = this.cache.get(model).get(groupsId);
    return this.mappers[id] = store.fn, store;
  }
  createContext(e) {
    const { cache: t } = this;
    return {
      ...e,
      cache: t
    };
  }
  compile(e, t, s = {}) {
    const r = isString(e) ? e : this.getType(e), i = this.getGroupsId(t);
    let a = this.cache.get(r) || this.cache.get(nameOf(r));
    if (a || (a = /* @__PURE__ */ new Map(), this.cache.set(r, a)), a.has("typeMapper"))
      return a.get("typeMapper");
    if (!a.has(i)) {
      const o = this.getId(r, i);
      a.set(i, {
        id: o
      });
      const n = s.mapper ? s.mapper(o, t) : this.createMapper(r, o, t);
      try {
        return this.eval(n, { id: o, model: r, groupsId: i });
      } catch (c) {
        throw new Error(`Fail to compile mapper for ${nameOf(e)}. See the error above: ${c.message}.
${n}`);
      }
    }
    return a.get(i);
  }
  execMapper(e, t, s) {
    return isObjectID(t) ? t.toString() : this.mappers[e || nameOf(classOf(t))](t, s);
  }
  getType(e) {
    if (!e)
      return Object;
    if (isClass(e) && !isCollection(e)) {
      const t = [Array, Map, Set].find((s) => ancestorsOf(e).includes(s));
      if (t)
        return t;
    }
    return e;
  }
  alterIgnore(e, t) {
    var r, i;
    let s = (i = (r = this.schemes[e]) == null ? void 0 : r.$hooks) == null ? void 0 : i.alter("ignore", !1, [t]);
    if (s)
      return s;
  }
  alterGroups(e, t) {
    return t !== !1 ? e.$hooks.alter("groups", !1, [t]) : !1;
  }
  getGroupsId(e) {
    return e === !1 ? "default" : e.length === 0 ? "-" : e.join(",");
  }
  getId(e, t) {
    return `${isString(e) ? e : nameOf(e)}:${getRandomId()}:${t}`;
  }
  getSchemaId(e, t) {
    return `${e}:${t}`;
  }
}
export {
  JsonMapperCompiler
};
//# sourceMappingURL=JsonMapperCompiler.js.map
